@@ -298,8 +298,8 @@ static Flight extractBoardingPass(KPkPass::Pass *pass, Flight flight)
     auto depGeo = depStation.geo();
     if (pass->locations().size() == 1 && !depGeo.isValid()) {
         const auto loc = pass->locations().at(0);
-        depGeo.setLatitude((float)loc.latitude());
-        depGeo.setLongitude((float)loc.longitude());
+        depGeo.setLatitude(loc.latitude());
+        depGeo.setLongitude(loc.longitude());
         depStation.setGeo(depGeo);
         trip.setDepartureStation(depStation);
     }
@@ -354,7 +354,7 @@ static void extractEventTicketPass(KPkPass::Pass *pass, EventReservation &eventR
     std::vector<PriceFinder::Result> prices;
     const auto fields = pass->fields();
     for (const auto &field : fields) {
-        priceFinder.findAll(field.value().toString(), prices);
+        priceFinder.findAll(field.valueDisplayString(), prices);
     }
     if (const auto price = priceFinder.highest(prices); price.hasResult()) {
         eventRes.setTotalPrice(price.value);
