@@ -8,13 +8,16 @@
 #include "logging.h"
 #include "rct2ticket.h"
 #include "uic9183block.h"
+#include "uic9183flex.h"
 #include "uic9183head.h"
 #include "uic9183header.h"
 #include "uic9183ticketlayout.h"
+#include "variantvisitor_p.h"
 #include "vendor0080block.h"
 #include "vendor1154block.h"
 
 #include "era/fcbticket.h"
+#include "era/fcbticket3.h"
 #include "era/fcbutil.h"
 
 #include <QDateTime>
@@ -78,7 +81,7 @@ QVariant Uic9183Parser::block(const QString &name) const
 
     BLOCK_FROM_NAME(Uic9183Head)
     BLOCK_FROM_NAME(Uic9183TicketLayout)
-    BLOCK_FROM_NAME(Fcb::UicRailTicketData)
+    BLOCK_FROM_NAME(Uic9183Flex)
     BLOCK_FROM_NAME(Vendor0080BLBlock)
     BLOCK_FROM_NAME(Vendor0080VUBlock)
     BLOCK_FROM_NAME(Vendor1154UTBlock)
@@ -180,20 +183,16 @@ QString Uic9183Parser::pnr() const
         return key;
     }
 
-    if (const auto fcb = findBlock<Fcb::UicRailTicketData>(); fcb.isValid()) {
-        if (!fcb.issuingDetail.issuerPNR.isEmpty()) {
-            return QString::fromLatin1(fcb.issuingDetail.issuerPNR);
+    if (const auto flex = findBlock<Uic9183Flex>(); flex.isValid()) {
+        QString pnr = std::visit([](auto &&fcb) { return QString::fromLatin1(fcb.issuingDetail.issuerPNR); }, flex.fcb());
+        if (!pnr.isEmpty()) {
+            return pnr;
         }
-        if (!fcb.transportDocument.isEmpty()) {
-            const auto doc = fcb.transportDocument.at(0);
-            QString pnr;
-            if (doc.ticket.userType() == qMetaTypeId<Fcb::ReservationData>()) {
-                pnr = fcbReference(doc.ticket.value<Fcb::ReservationData>());
-            } else if (doc.ticket.userType() == qMetaTypeId<Fcb::OpenTicketData>()) {
-                pnr = fcbReference(doc.ticket.value<Fcb::OpenTicketData>());
-            } else if (doc.ticket.userType() == qMetaTypeId<Fcb::PassData>()) {
-                pnr = fcbReference(doc.ticket.value<Fcb::PassData>());
-            }
+        if (flex.hasTransportDocument()) {
+            const auto doc = flex.transportDocuments().at(0);
+            QString pnr = VariantVisitor([](auto &&data) {
+                return fcbReference(data);
+            }).visit<Fcb::v13::ReservationData, Fcb::v13::OpenTicketData, Fcb::v13::PassData, Fcb::v3::ReservationData, Fcb::v3::OpenTicketData, Fcb::v3::PassData>(doc);
             if (!pnr.isEmpty()) {
                 return pnr;
             }
@@ -203,28 +202,14 @@ QString Uic9183Parser::pnr() const
     return {};
 }
 
-template <typename T>
-static QString fcbTariffName(const T &data)
-{
-    if (data.tariffs.isEmpty()) {
-        return {};
-    }
-    return data.tariffs.at(0).tariffDesc;
-}
-
 QString Uic9183Parser::name() const
 {
     // ERA FCB
-    if (const auto fcb = findBlock<Fcb::UicRailTicketData>(); fcb.isValid() && !fcb.transportDocument.isEmpty()) {
-        const auto doc = fcb.transportDocument.at(0);
-        QString name;
-        if (doc.ticket.userType() == qMetaTypeId<Fcb::ReservationData>()) {
-            name = fcbTariffName(doc.ticket.value<Fcb::ReservationData>());
-        } else if (doc.ticket.userType() == qMetaTypeId<Fcb::OpenTicketData>()) {
-            name = fcbTariffName(doc.ticket.value<Fcb::OpenTicketData>());
-        } else if (doc.ticket.userType() == qMetaTypeId<Fcb::PassData>()) {
-            name = fcbTariffName(doc.ticket.value<Fcb::PassData>());
-        }
+    if (const auto flex = findBlock<Uic9183Flex>(); flex.hasTransportDocument()) {
+        const auto doc = flex.transportDocuments().at(0);
+        QString name = VariantVisitor([](auto &&data) {
+            return data.tariffs.isEmpty() ? QString() : data.tariffs.at(0).tariffDesc;
+        }).visit<Fcb::v13::ReservationData, Fcb::v13::OpenTicketData, Fcb::v13::PassData, Fcb::v3::ReservationData, Fcb::v3::OpenTicketData, Fcb::v3::PassData>(doc);
         if (!name.isEmpty()) {
             return name;
         }
@@ -251,13 +236,18 @@ QString Uic9183Parser::carrierId() const
     if (const auto head = findBlock<Uic9183Head>(); head.isValid()) {
         return head.issuerCompanyCodeString();
     }
-    if (const auto fcb = findBlock<Fcb::UicRailTicketData>(); fcb.isValid()) {
-        const auto issue = fcb.issuingDetail;
-        if (issue.issuerNumIsSet()) {
-            return QString::number(issue.issuerNum);
-        }
-        if (issue.issuerIA5IsSet()) {
-            return QString::fromLatin1(issue.issuerIA5);
+    if (const auto flex = findBlock<Uic9183Flex>(); flex.isValid()) {
+        QString id = std::visit([](auto &&fcb) {
+            if (fcb.issuingDetail.issuerNumIsSet()) {
+                return QString::number(fcb.issuingDetail.issuerNum);
+            }
+            if (fcb.issuingDetail.issuerIA5IsSet()) {
+                return QString::fromLatin1(fcb.issuingDetail.issuerIA5);
+            }
+            return QString();
+        }, flex.fcb());
+        if (!id.isEmpty()) {
+            return id;
         }
     }
     return header().signerCompanyCode();
@@ -267,8 +257,12 @@ Organization Uic9183Parser::issuer() const
 {
     Organization issuer;
     issuer.setIdentifier(QLatin1StringView("uic:") + carrierId());
-    if (const auto fcb = findBlock<Fcb::UicRailTicketData>(); fcb.isValid() && fcb.issuingDetail.issuerNameIsSet()) {
-        issuer.setName(fcb.issuingDetail.issuerName);
+    if (const auto flex = findBlock<Uic9183Flex>(); flex.isValid()) {
+        std::visit([&issuer](auto &&fcb) {
+            if (fcb.issuingDetail.issuerNameIsSet()) {
+                issuer.setName(fcb.issuingDetail.issuerName);
+            }
+        }, flex.fcb());
     }
     return issuer;
 }
@@ -276,17 +270,20 @@ Organization Uic9183Parser::issuer() const
 QDateTime Uic9183Parser::validFrom() const
 {
     // ERA FCB
-    if (const auto fcb = findBlock<Fcb::UicRailTicketData>(); fcb.isValid() && !fcb.transportDocument.isEmpty()) {
-        const auto issue = fcb.issuingDetail.issueingDateTime();
-        const auto doc = fcb.transportDocument.at(0).ticket;
-        if (doc.userType() == qMetaTypeId<Fcb::ReservationData>()) {
-            return doc.value<Fcb::ReservationData>().departureDateTime(issue);
+    if (const auto flex = findBlock<Uic9183Flex>(); flex.hasTransportDocument()) {
+        const auto issue = flex.issuingDateTime();
+        const auto doc = flex.transportDocuments().at(0);
+        auto dt = VariantVisitor([issue](auto &&data) {
+            return data.departureDateTime(issue);
+        }).visit<Fcb::v13::ReservationData, Fcb::v3::ReservationData>(doc);
+        if (dt.isValid()) {
+            return dt;
         }
-        if (doc.userType() == qMetaTypeId<Fcb::OpenTicketData>()) {
-            return doc.value<Fcb::OpenTicketData>().validFrom(issue);
-        }
-        if (doc.userType() == qMetaTypeId<Fcb::PassData>()) {
-            return doc.value<Fcb::PassData>().validFrom(issue);
+        dt = VariantVisitor([issue](auto &&data) {
+            return data.validFrom(issue);
+        }).visit<Fcb::v13::OpenTicketData, Fcb::v13::PassData, Fcb::v3::OpenTicketData, Fcb::v3::PassData>(doc);
+        if (dt.isValid()) {
+            return dt;
         }
     }
 
@@ -336,17 +333,20 @@ QDateTime Uic9183Parser::validFrom() const
 QDateTime Uic9183Parser::validUntil() const
 {
     // ERA FCB
-    if (const auto fcb = findBlock<Fcb::UicRailTicketData>(); fcb.isValid() && !fcb.transportDocument.isEmpty()) {
-        const auto issue = fcb.issuingDetail.issueingDateTime();
-        const auto doc = fcb.transportDocument.at(0).ticket;
-        if (doc.userType() == qMetaTypeId<Fcb::ReservationData>()) {
-            return doc.value<Fcb::ReservationData>().arrivalDateTime(issue);
+    if (const auto flex = findBlock<Uic9183Flex>(); flex.hasTransportDocument()) {
+        const auto issue = flex.issuingDateTime();
+        const auto doc = flex.transportDocuments().at(0);
+        auto dt = VariantVisitor([issue](auto &&data) {
+            return data.arrivalDateTime(issue);
+        }).visit<Fcb::v13::ReservationData, Fcb::v3::ReservationData>(doc);
+        if (dt.isValid()) {
+            return dt;
         }
-        if (doc.userType() == qMetaTypeId<Fcb::OpenTicketData>()) {
-            return doc.value<Fcb::OpenTicketData>().validUntil(issue);
-        }
-        if (doc.userType() == qMetaTypeId<Fcb::PassData>()) {
-            return doc.value<Fcb::PassData>().validUntil(issue);
+        dt = VariantVisitor([issue](auto &&data) {
+            return data.validUntil(issue);
+        }).visit<Fcb::v13::OpenTicketData, Fcb::v13::PassData, Fcb::v3::OpenTicketData, Fcb::v3::PassData>(doc);
+        if (dt.isValid()) {
+            return dt;
         }
     }
 
@@ -395,14 +395,23 @@ QDateTime Uic9183Parser::validUntil() const
 Person Uic9183Parser::person() const
 {
     // ERA FCB
-    if (const auto fcb = findBlock<Fcb::UicRailTicketData>(); fcb.isValid() && fcb.travelerDetailIsSet() && fcb.travelerDetail.traveler.size() == 1) {
-        const auto traveler = fcb.travelerDetail.traveler.at(0);
+    if (const auto flex = findBlock<Uic9183Flex>(); flex.isValid()) {
+        bool travelerFound = false;
         Person p;
-        if (traveler.firstNameIsSet() || traveler.secondNameIsSet()) {
-            p.setGivenName(QString(traveler.firstName + QLatin1Char(' ') + traveler.secondName).trimmed());
-        }
-        p.setFamilyName(traveler.lastName);
-        if (traveler.firstNameIsSet() || traveler.lastNameIsSet()) {
+        std::visit([&p, &travelerFound](auto &&fcb) {
+            if (!fcb.travelerDetailIsSet() || fcb.travelerDetail.traveler.size() != 1) {
+                return;
+            }
+            const auto traveler = fcb.travelerDetail.traveler.at(0);
+            if (traveler.firstNameIsSet() || traveler.secondNameIsSet()) {
+                p.setGivenName(QString(traveler.firstName + QLatin1Char(' ') + traveler.secondName).trimmed());
+            }
+            p.setFamilyName(traveler.lastName);
+            if (traveler.firstNameIsSet() || traveler.lastNameIsSet()) {
+                travelerFound = true;
+            }
+        }, flex.fcb());
+        if (travelerFound) {
             return p;
         }
     }
@@ -454,19 +463,6 @@ Person Uic9183Parser::person() const
     return {};
 }
 
-static void fixFcbStationCode(TrainStation &station)
-{
-    // UIC codes in Germany are wildly unreliable, there seem to be different
-    // code tables in use by different operators, so we unfortunately have to ignore
-    // those entirely
-    if (station.identifier().startsWith(QLatin1StringView("uic:80"))) {
-      PostalAddress addr;
-      addr.setAddressCountry(QStringLiteral("DE"));
-      station.setAddress(addr);
-      station.setIdentifier(QString());
-    }
-}
-
 TrainStation Uic9183Parser::outboundDepartureStation() const
 {
     TrainStation station;
@@ -490,18 +486,8 @@ TrainStation Uic9183Parser::outboundDepartureStation() const
     }
 
     // ERA FCB
-    if (const auto fcb = findBlock<Fcb::UicRailTicketData>(); fcb.isValid() && !fcb.transportDocument.isEmpty()) {
-        const auto doc = fcb.transportDocument.at(0);
-        if (doc.ticket.userType() == qMetaTypeId<Fcb::ReservationData>()) {
-            const auto irt = doc.ticket.value<Fcb::ReservationData>();
-            station.setName(irt.fromStationNameUTF8);
-            station.setIdentifier(FcbUtil::fromStationIdentifier(irt));
-        } else if (doc.ticket.userType() == qMetaTypeId<Fcb::OpenTicketData>()) {
-            const auto nrt = doc.ticket.value<Fcb::OpenTicketData>();
-            station.setName(nrt.fromStationNameUTF8);
-            station.setIdentifier(FcbUtil::fromStationIdentifier(nrt));
-        }
-        fixFcbStationCode(station);
+    if (const auto flex = findBlock<Uic9183Flex>(); flex.hasTransportDocument()) {
+        Uic9183Flex::readDepartureStation(flex.transportDocuments().at(0), station);
     }
 
     return station;
@@ -529,18 +515,8 @@ TrainStation Uic9183Parser::outboundArrivalStation() const
     }
 
     // ERA FCB
-    if (const auto fcb = findBlock<Fcb::UicRailTicketData>(); fcb.isValid() && !fcb.transportDocument.isEmpty()) {
-        const auto doc = fcb.transportDocument.at(0);
-        if (doc.ticket.userType() == qMetaTypeId<Fcb::ReservationData>()) {
-            const auto irt = doc.ticket.value<Fcb::ReservationData>();
-            station.setName(irt.toStationNameUTF8);
-            station.setIdentifier(FcbUtil::toStationIdentifier(irt));
-        } else if (doc.ticket.userType() == qMetaTypeId<Fcb::OpenTicketData>()) {
-            const auto nrt = doc.ticket.value<Fcb::OpenTicketData>();
-            station.setName(nrt.toStationNameUTF8);
-            station.setIdentifier(FcbUtil::toStationIdentifier(nrt));
-        }
-        fixFcbStationCode(station);
+    if (const auto flex = findBlock<Uic9183Flex>(); flex.hasTransportDocument()) {
+        Uic9183Flex::readArrivalStation(flex.transportDocuments().at(0), station);
     }
 
     return station;
@@ -567,10 +543,9 @@ TrainStation Uic9183Parser::returnDepartureStation() const
     }
 
     // ERA FCB
-    if (const auto fcb = findBlock<Fcb::UicRailTicketData>(); fcb.isValid() && !fcb.transportDocument.isEmpty()) {
-        const auto doc = fcb.transportDocument.at(0);
-        if (doc.ticket.userType() == qMetaTypeId<Fcb::OpenTicketData>()) {
-            const auto nrt = doc.ticket.value<Fcb::OpenTicketData>();
+    if (const auto flex = findBlock<Uic9183Flex>(); flex.hasTransportDocument()) {
+        const auto doc = flex.transportDocuments().at(0);
+        VariantVisitor([&station, outboundArrival](auto &&nrt) {
             if (nrt.returnIncluded && nrt.returnDescriptionIsSet()) {
                 station.setName(nrt.returnDescription.fromStationNameUTF8);
                 station.setIdentifier(FcbUtil::fromStationIdentifier(nrt.stationCodeTable, nrt.returnDescription));
@@ -579,8 +554,8 @@ TrainStation Uic9183Parser::returnDepartureStation() const
                     station.setIdentifier(outboundArrival.identifier());
                 }
             }
-        }
-        fixFcbStationCode(station);
+        }).visit<Fcb::v13::OpenTicketData, Fcb::v3::OpenTicketData>(doc);
+        Uic9183Flex::fixStationCode(station);
     }
 
     return station;
@@ -607,10 +582,9 @@ TrainStation Uic9183Parser::returnArrivalStation() const
     }
 
     // ERA FCB
-    if (const auto fcb = findBlock<Fcb::UicRailTicketData>(); fcb.isValid() && !fcb.transportDocument.isEmpty()) {
-        const auto doc = fcb.transportDocument.at(0);
-        if (doc.ticket.userType() == qMetaTypeId<Fcb::OpenTicketData>()) {
-            const auto nrt = doc.ticket.value<Fcb::OpenTicketData>();
+    if (const auto flex = findBlock<Uic9183Flex>(); flex.hasTransportDocument()) {
+        const auto doc = flex.transportDocuments().at(0);
+        VariantVisitor([&station, outboundDeparture](auto &&nrt) {
             if (nrt.returnIncluded && nrt.returnDescriptionIsSet()) {
                 station.setName(nrt.returnDescription.toStationNameUTF8);
                 station.setIdentifier(FcbUtil::toStationIdentifier(nrt.stationCodeTable, nrt.returnDescription));
@@ -619,8 +593,8 @@ TrainStation Uic9183Parser::returnArrivalStation() const
                     station.setIdentifier(outboundDeparture.identifier());
                 }
             }
-        }
-        fixFcbStationCode(station);
+        }).visit<Fcb::v13::OpenTicketData, Fcb::v3::OpenTicketData>(doc);
+        Uic9183Flex::fixStationCode(station);
     }
 
     return station;
@@ -628,16 +602,13 @@ TrainStation Uic9183Parser::returnArrivalStation() const
 
 QString Uic9183Parser::seatingType() const
 {
-    if (const auto fcb = findBlock<Fcb::UicRailTicketData>(); fcb.isValid() && fcb.transportDocument.size() == 1) {
-        const auto doc = fcb.transportDocument.at(0);
-        if (doc.ticket.userType() == qMetaTypeId<Fcb::ReservationData>()) {
-            return FcbUtil::classCodeToString(doc.ticket.value<Fcb::ReservationData>().classCode);
-        }
-        if (doc.ticket.userType() == qMetaTypeId<Fcb::OpenTicketData>()) {
-            return FcbUtil::classCodeToString(doc.ticket.value<Fcb::OpenTicketData>().classCode);
-        }
-        if (doc.ticket.userType() == qMetaTypeId<Fcb::PassData>()) {
-            return FcbUtil::classCodeToString(doc.ticket.value<Fcb::PassData>().classCode);
+    if (const auto flex = findBlock<Uic9183Flex>(); flex.transportDocuments().size() == 1) {
+        const auto doc = flex.transportDocuments().at(0);
+        auto c = VariantVisitor([](auto &&data) {
+            return FcbUtil::classCodeToString(data.classCode);
+        }).visit<Fcb::v13::ReservationData, Fcb::v13::OpenTicketData, Fcb::v13::PassData, Fcb::v3::ReservationData, Fcb::v3::OpenTicketData, Fcb::v3::PassData>(doc);
+        if (!c.isEmpty()) {
+            return c;
         }
     }
 
